#include <occupancy_grid_utils/ray_tracer.h>
#include <occupancy_grid_utils/exceptions.h>
#include <boost/bind.hpp>
#include <boost/ref.hpp>
#include <algorithm>

namespace occupancy_grid_utils
{

namespace nm=nav_msgs;
namespace gm=geometry_msgs;

using boost::bind;
using boost::ref;


RayTraceIterRange rayTrace (const Cell& c1, const Cell& c2)
{
  return RayTraceIterRange(RayTraceIterator(c1,c2), RayTraceIterator(c2,c2,true));
}

inline
bool cellWithinBounds (const nm::MapMetaData& info, const Cell& c)
{
  return withinBounds(info, c);
}

Cell rayTraceOntoGrid (const nm::MapMetaData& info, const Cell& c1, const Cell& c2)
{
  ROS_ASSERT(withinBounds(info, c1));
  RayTraceIterRange r = rayTrace(c2, c1);
  RayTraceIterator pos = std::find_if (r.first, r.second, bind(cellWithinBounds, ref(info), _1));
  ROS_ASSERT (pos!=r.second);
  return *pos;
}

RayTraceIterRange rayTrace (const nm::MapMetaData& info, const gm::Point& p1, const gm::Point& p2,
                            bool project_onto_grid)
{
  const Cell c1 = pointCell(info, p1);
  if (!withinBounds(info, c1))
    throw PointOutOfBoundsException(p1);
  Cell c2 = pointCell(info, p2);
  if (!withinBounds(info, p2)) {
    if (project_onto_grid) {
      c2 = rayTraceOntoGrid(info, c1, c2);
    }
    else {
      throw PointOutOfBoundsException(p2);
    }
  }

  return rayTrace(c1, c2);
}





} // namespace occupancy_grid_utils
