/*
 * Copyright (c) 2008, Willow Garage, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Willow Garage, Inc. nor the names of its
 *       contributors may be used to endorse or promote products derived from
 *       this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */


/**
 * \file
 *
 * Definition of FakeLoopClosureConstraintGenerator
 *
 * \author Bhaskara Marthi
 */

#include <pose_graph/constraint_generator.h>
#include <geometry_msgs/PoseWithCovarianceStamped.h>
#include <ros/ros.h>
#include <boost/optional.hpp>

namespace pose_graph
{

const double LOCALIZATION_RECENCY_THRESHOLD = 1.0;

class FakeLoopClosureGenerator: public ConstraintGenerator
{
public:

  FakeLoopClosureGenerator (double proximity_threshold);
  NodeConstraintVector getConstraints (const PoseGraph& graph, const NodeId new_node,
                                       const NodePoseMap& optimized_poses);

private:

  void locCB (const geometry_msgs::PoseWithCovarianceStamped& msg);
  bool haveRecentlyReceivedPose () const;

  /****************************************
   * Associated objects
   ****************************************/

  ros::NodeHandle nh_;
  ros::Subscriber fake_loc_sub_;

/****************************************
 * Params
 ****************************************/
  
  const double proximity_threshold_;

  /****************************************
   * State
   ****************************************/

  boost::optional<geometry_msgs
::PoseStamped> last_pose_;
  NodePoseMap true_poses_;
  
};


} // namespace pose_graph


