/*
 * Copyright (c) 2008, Willow Garage, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Willow Garage, Inc. nor the names of its
 *       contributors may be used to endorse or promote products derived from
 *       this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */

/**
 * \file 
 * 
 * Finds loop closure links using karto scan matcher
 *
 * \author Bhaskara Marthi
 */

#ifndef POSE_GRAPH_LOOP_SCAN_MATCHER_H
#define POSE_GRAPH_LOOP_SCAN_MATCHER_H

#include <pose_graph/scan_manager.h>

namespace pose_graph
{

class LoopScanMatcher : public ScanManager
{
public:

  LoopScanMatcher (const geometry_msgs::Pose2D& laser_offset, double max_link_length,
                   unsigned min_chain_length, double min_response);

  NodeConstraintVector getConstraints (const PoseGraph& g, const NodeId ref_node,
                                       const Pose& init_pose_estimate, const LaserPtr scan) const;

  ChainVec possibleLoopClosures (const PoseGraph& graph, NodeId ref_node,
                                 const Pose& initial_pose_estimate, const LaserPtr scan) const;

private:

  bool furtherThan (const double max_distance, const geometry_msgs::Point& p, const NodeId n) const;

  // Mutable because it is initialized in getConstraints, which is logically const
  mutable KartoMatcherPtr matcher_; 

  const double max_link_length_;
  const double min_chain_length_;
  const double min_response_;

};

} // namespace

#endif // include guard
