/*
 * Copyright (c) 2008, Willow Garage, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Willow Garage, Inc. nor the names of its
 *       contributors may be used to endorse or promote products derived from
 *       this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */

/**
 * \file
 * 
 * Code pertaining to converting between pose graphs and ros messages
 */


#ifndef POSE_GRAPH_POSE_GRAPH_MESSAGE_H
#define POSE_GRAPH_POSE_GRAPH_MESSAGE_H

#include <pose_graph/pose_graph.h>
#include <pose_graph/PoseGraphMessage.h>
#include <pose_graph/PoseGraphDiff.h>
#include <string>


namespace pose_graph
{

using std::string;


/// Converts a PoseGraph into a PoseGraphMessage
PoseGraphMessage poseGraphToRos (const PoseGraph& g);

/// Converts a PoseGraphMessage into a PoseGraph
PoseGraph poseGraphFromRos (const PoseGraphMessage& g);

/// \brief Writes a PoseGraph to a file
/// \throws FileOpenException
void writeToFile (const PoseGraph& g, const string& filename);

/// \brief Reads a PoseGraph from a file
/// \throws FileOpenException
PoseGraph readFromFile (const string& filename);
  
/// Helper for converting an individual constraint to a ros message 
PoseWithPrecision constraintToRos (const PoseConstraint& constraint);

/// Helper for converting a ros message to a PoseConstraint
PoseConstraint constraintFromRos (const PoseWithPrecision& m);

/// Apply a diff to a pose graph
void applyDiff (PoseGraph* g, const PoseGraphDiff& m);

} // namespace pose_graph

#endif // POSE_GRAPH_POSE_GRAPH_MESSAGE_H
