/*
 * Copyright (c) 2008, Willow Garage, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Willow Garage, Inc. nor the names of its
 *       contributors may be used to endorse or promote products derived from
 *       this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */


/**
 * \file
 *
 * Defines ScanMatchLocalization
 *
 * \author Bhaskara Marthi
 */

#ifndef POSE_GRAPH_SCAN_MATCH_LOCALIZATION_H
#define POSE_GRAPH_SCAN_MATCH_LOCALIZATION_H

#include <pose_graph/scan_manager.h>
#include <pose_graph/transforms.h>


namespace pose_graph
{

struct LocalizationState
{
  LocalizationState (const NodeId n, const Eigen::Transform3d trans, const Pose& pose) :
    ref_node(n), node_to_fixed_frame(trans), node_frame_pose(pose) {}
  
  NodeId ref_node;
  Eigen::Transform3d node_to_fixed_frame;
  Pose node_frame_pose;
};

class ScanMatchLocalization : public ScanManager
{
public:

  /// \brief Constructor
  /// \param laser_offset Pose of laser in base frame
  /// \param neighborhood_size Size (m) of neighborhood of current node used for scan matching
  ScanMatchLocalization (const geometry_msgs::Pose2D& laser_offset, double neighborhood_size);
                         
  /// \brief Compute the next state given the current state, odom pose, scan, graph, and optimized poses
  LocalizationState nextState (const LocalizationState& state,
                               const Pose& fixed_frame_pose, const LaserPtr scan, 
                               const PoseGraph& graph, const NodePoseMap& optimized_poses) const;

private:

  const double max_link_length_;

  // Mutable because it is initialized in nextState, which is logically const
  mutable KartoMatcherPtr matcher_;

};


  

  
  

} // namespace pose_graph

#endif // include guard
